<?php
/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

?>
<?php
	/**
	 * Task ItemModule
	 * Module which openes, creates, saves and deletes an item. It 
	 * extends the Module class.
	 */
	class TaskItemModule extends ItemModule
	{
		var $plaintext;

		/**
		 * Constructor
		 * @param int $id unique id.
		 * @param array $data list of all actions.
		 */
		function TaskItemModule($id, $data)
		{
			$this->properties = $GLOBALS["properties"]->getTaskProperties();
			
			$this->plaintext = true;

			parent::ItemModule($id, $data);
		}

		/**
		 * Function which saves an item.
		 * @param object $store MAPI Message Store Object
		 * @param string $parententryid parent entryid of the message
		 * @param array $action the action data, sent by the client
		 * @return boolean true on success or false on failure		 		 
		 */
		function save($store, $parententryid, $action)
		{
			$result = false;
			
			if(isset($action["props"])) {
				$props = $action["props"];
				
				if(!$store && !$parententryid) {
					if(isset($action["props"]["message_class"])) {
						$store = $GLOBALS["mapisession"]->getDefaultMessageStore();
						$parententryid = $this->getDefaultFolderEntryID($store, $action["props"]["message_class"]);
					}
				}

				if($store && $parententryid) {
					// Fetch message store properties of a current store.
					$msgstoreProps = mapi_getprops($store, array(PR_USER_ENTRYID, PR_MDB_PROVIDER, PR_MAILBOX_OWNER_ENTRYID));

					// Get current session and open addressbook
					$addrbook = $GLOBALS["mapisession"]->getAddressbook();
					//$addrbook = mapi_openaddressbook($session);

					// Open addressbook entry for current user.
					$userObject = mapi_ab_openentry($addrbook, $msgstoreProps[PR_USER_ENTRYID]);

					$props = $action["props"];

					if(mapi_last_hresult() == NOERROR){
						$userProps = mapi_getprops($userObject);
						if(mapi_last_hresult() == NOERROR){
							// Store PR_SENDER_* properties for task in props variable.
							$props["sender_email_address"] = $userProps[PR_EMAIL_ADDRESS];
							$props["sender_name"] = $userProps[PR_DISPLAY_NAME];
							$props["sender_addrtype"] = $userProps[PR_ADDRTYPE];
							$props["sender_entryid"] = bin2hex($userProps[PR_ENTRYID]);
							$props["sender_search_key"] = $userProps[PR_SEARCH_KEY];
						}
					}

					// Store PR_SENT_REPRESENTING_* properties for task according to store type.
					switch ($msgstoreProps[PR_MDB_PROVIDER])
					{
						case ZARAFA_STORE_PUBLIC_GUID:
							/**
							 * store type is "public"
							 * Store PR_SENT_REPRESENTING_* properties for task in props variable.
							 */
							$props["sent_representing_entryid"] = bin2hex($userProps[PR_ENTRYID]);
							$props["sent_representing_name"] = $userProps[PR_DISPLAY_NAME];
							$props["sent_representing_addrtype"] = $userProps[PR_ADDRTYPE];
							$props["sent_representing_email_address"] = $userProps[PR_EMAIL_ADDRESS];
							$props["sent_representing_search_key"] = $userProps[PR_SEARCH_KEY];
							break;
						case ZARAFA_SERVICE_GUID:
							// store type is "default"
						case ZARAFA_STORE_DELEGATE_GUID:
							/**
							 * store type is "delegate"
							 * Open addressbook entry for mailbox owner.
							 */
							$ownerObject = mapi_ab_openentry($addrbook, $msgstoreProps[PR_MAILBOX_OWNER_ENTRYID]);
							if(mapi_last_hresult() == NOERROR){
								$ownerProps = mapi_getprops($ownerObject);
								if(mapi_last_hresult() == NOERROR){
									// Store PR_SENT_REPRESENTING_* properties for task in props variable.
									$props["sent_representing_entryid"] = bin2hex($ownerProps[PR_ENTRYID]);
									$props["sent_representing_name"] = $ownerProps[PR_DISPLAY_NAME];
									$props["sent_representing_addrtype"] = $ownerProps[PR_ADDRTYPE];
									$props["sent_representing_email_address"] = $ownerProps[PR_EMAIL_ADDRESS];
									$props["sent_representing_search_key"] = $ownerProps[PR_SEARCH_KEY];
								}
							}
							break;
					}

					$messageProps = array(); // returned props
					$result = $GLOBALS["operations"]->saveMessage($store, $parententryid, Conversion::mapXML2MAPI($this->properties, $props), false, (isset($action["dialog_attachments"])?$action["dialog_attachments"]:null), $messageProps);
					
					if($result) {
						$GLOBALS["bus"]->notify(bin2hex($parententryid), TABLE_SAVE, $messageProps);
					}
				}
			}
			
			return $result;
		}
	}
?>
